==> core-android-debug-1.30.0-sources.jar: androidMain/io/dyte/core/VideoView.kt <==
package io.dyte.core

import android.content.Context
import android.util.AttributeSet
import android.widget.LinearLayout
import io.dyte.core.controllers.IControllerContainer
import io.dyte.core.models.DyteJoinedMeetingParticipant
import io.dyte.core.observability.DyteLogger
import io.dyte.webrtc.VideoStreamTrack
import io.dyte.webrtc.WebRtc
import io.webrtc.EglBase
import io.webrtc.RendererCommon.ScalingType.SCALE_ASPECT_FIT
import io.webrtc.SurfaceViewRenderer
import io.webrtc.VideoSink

class VideoView : LinearLayout {
  private var participant: DyteJoinedMeetingParticipant? = null
  var videoRenderer: SurfaceViewRenderer? = null
  private var controllerContainer: IControllerContainer? = null

  private var isScreenShare: Boolean = false

  constructor(context: Context) : super(context) {
    init(context)
  }

  constructor(context: Context, attrs: AttributeSet?) : super(context, attrs) {
    init(context)
  }

  constructor(
    context: Context,
    attrs: AttributeSet?,
    defStyleAttr: Int
  ) : super(context, attrs, defStyleAttr) {
    init(context)
  }

  private fun init(context: Context) {
    inflate(context, R.layout.view_video, this)
    videoRenderer = findViewById(R.id.video_renderer)
    addOnLayoutChangeListener { view, i, i2, i3, i4, i5, i6, i7, i8 ->
      DyteLogger.info("VideoView::onLayoutchanged::")
      stopVideoRender()
      renderVideo()
    }
  }

  internal fun setup(
    controllerContainer: IControllerContainer,
    dyteParticipant: DyteJoinedMeetingParticipant,
    isScreenShare: Boolean
  ) {
    this.participant = dyteParticipant
    this.controllerContainer = controllerContainer
    this.isScreenShare = isScreenShare
    activate()
  }

  private fun activate() {
    videoRenderer?.setScalingType(SCALE_ASPECT_FIT)
    try {
      videoRenderer?.init(WebRtc.rootEglBase.eglBaseContext as EglBase.Context, null)
    } catch (e: Exception) {
      DyteLogger.error("VideoView::activate::error", e)
    }
  }

  fun renderVideo() {
    try {
      val track = getVideoTrack()

      if (track != null) {
        videoRenderer?.visibility = VISIBLE
        track.addSinkCatching(videoRenderer!!)
      } else {
        videoRenderer?.visibility = GONE
      }
    } catch (e: Exception) {
      DyteLogger.error("VideoView::renderVideo::error", e)
    }
  }

  fun stopVideoRender() {
    try {
      val track: VideoStreamTrack? = getVideoTrack()

      if (track != null) {
        videoRenderer?.visibility = VISIBLE
        track.removeSinkCatching(videoRenderer!!)
      } else {
        videoRenderer?.visibility = GONE
      }
    } catch (e: Exception) {
      e.printStackTrace()
    }
  }

  fun release() {
    val track = getVideoTrack()
    track?.removeSinkCatching(videoRenderer!!)
    videoRenderer!!.release()
  }

  private fun getVideoTrack(): VideoStreamTrack? {
    var track: VideoStreamTrack? = null
    if (isScreenShare && participant?.screenShareTrack != null) {
      track = participant?.screenShareTrack!!
    } else if (participant?.videoEnabled == true && participant?.videoTrack != null) {
      track = participant?.videoTrack
    }
    return track
  }

  private fun VideoStreamTrack.addSinkCatching(sink: VideoSink) {
    // runCatching as track may be disposed while activity was in pause mode
    runCatching { addSink(sink) }
  }

  private fun VideoStreamTrack.removeSinkCatching(sink: VideoSink) {
    // runCatching as track may be disposed while activity was in pause mode
    runCatching { removeSink(sink) }
  }
}

==> core-android-debug-1.32.2-sources.jar: androidMain/io/dyte/core/VideoView.kt <==
package io.dyte.core

import android.content.Context
import android.util.AttributeSet
import android.widget.LinearLayout
import io.dyte.core.controllers.IControllerContainer
import io.dyte.core.models.DyteJoinedMeetingParticipant
import io.dyte.core.observability.DyteLogger
import io.dyte.webrtc.VideoStreamTrack
import io.dyte.webrtc.WebRtc
import io.webrtc.EglBase
import io.webrtc.RendererCommon.ScalingType.SCALE_ASPECT_FIT
import io.webrtc.SurfaceViewRenderer
import io.webrtc.VideoSink

class VideoView : LinearLayout {
  private var participant: DyteJoinedMeetingParticipant? = null
  var videoRenderer: SurfaceViewRenderer? = null
  private var controllerContainer: IControllerContainer? = null

  private var isScreenShare: Boolean = false

  constructor(context: Context) : super(context) {
    init(context)
  }

  constructor(context: Context, attrs: AttributeSet?) : super(context, attrs) {
    init(context)
  }

  constructor(
    context: Context,
    attrs: AttributeSet?,
    defStyleAttr: Int,
  ) : super(context, attrs, defStyleAttr) {
    init(context)
  }

  private fun init(context: Context) {
    inflate(context, R.layout.view_video, this)
    videoRenderer = findViewById(R.id.video_renderer)
    addOnLayoutChangeListener { view, i, i2, i3, i4, i5, i6, i7, i8 ->
      stopVideoRender()
      renderVideo()
    }
  }

  internal fun setup(
    controllerContainer: IControllerContainer,
    dyteParticipant: DyteJoinedMeetingParticipant,
    isScreenShare: Boolean,
  ) {
    this.participant = dyteParticipant
    this.controllerContainer = controllerContainer
    this.isScreenShare = isScreenShare
    activate()
  }

  private fun activate() {
    videoRenderer?.setScalingType(SCALE_ASPECT_FIT)
    try {
      videoRenderer?.init(WebRtc.rootEglBase.eglBaseContext as EglBase.Context, null)
    } catch (e: Exception) {
      DyteLogger.error("VideoView::activate::error", e)
    }
  }

  fun renderVideo() {
    try {
      val track = getVideoTrack()

      if (track != null) {
        videoRenderer?.visibility = VISIBLE
        track.addSinkCatching(videoRenderer!!)
      } else {
        videoRenderer?.visibility = GONE
      }
    } catch (e: Exception) {
      DyteLogger.error("VideoView::renderVideo::error", e)
    }
  }

  fun stopVideoRender() {
    try {
      val track: VideoStreamTrack? = getVideoTrack()

      if (track != null) {
        videoRenderer?.visibility = VISIBLE
        track.removeSinkCatching(videoRenderer!!)
      } else {
        videoRenderer?.visibility = GONE
      }
    } catch (e: Exception) {
      e.printStackTrace()
    }
  }

  fun release() {
    val track = getVideoTrack()
    track?.removeSinkCatching(videoRenderer!!)
    videoRenderer!!.release()
  }

  private fun getVideoTrack(): VideoStreamTrack? {
    var track: VideoStreamTrack? = null
    if (isScreenShare && participant?.screenShareTrack != null) {
      track = participant?.screenShareTrack!!
    } else if (participant?.videoEnabled == true && participant?.videoTrack != null) {
      track = participant?.videoTrack
    }
    return track
  }

  private fun VideoStreamTrack.addSinkCatching(sink: VideoSink) {
    // runCatching as track may be disposed while activity was in pause mode
    runCatching { addSink(sink) }
  }

  private fun VideoStreamTrack.removeSinkCatching(sink: VideoSink) {
    // runCatching as track may be disposed while activity was in pause mode
    runCatching { removeSink(sink) }
  }
}
